#include "general.h"
#include "GObject.h"
#include "Window.h"
#include "SizeW.h"
#include "Console.h"
#include "Button.h"

namespace ns_console
{
	int hide(GtkWidget *w)
	{
		gtk_widget_hide_all(w);
		return true;
	}

	int show_size_dialog(GtkWidget *w, Console *c)
	{
		c->getSizeW()->show();
		return true;
	}
};

Console::Console(int lines):
	_window(480,180),
	_sw(*this)
{
	_lines = lines;
	Button *close = new Button("Button_Close", " Close ");
	Button *save = new Button("Button_Size", " Size ");
	
	_window.addWidget(close);
	_window.addWidget(save);
	
	GtkWidget *vbox = _window.getWidget("vbox");
	
	GtkWidget *txt_hbox = gtk_hbox_new(FALSE, 2);
	GtkWidget *text = gtk_text_new(NULL, NULL);
	GtkWidget *vscrollbar = gtk_vscrollbar_new(GTK_TEXT(text)->vadj);
	gtk_box_pack_start(GTK_BOX(txt_hbox), text, TRUE, TRUE, 0);
	gtk_box_pack_start(GTK_BOX(txt_hbox), vscrollbar, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), txt_hbox, TRUE, TRUE, 0);
	
	_window.addWidget("text_scrollbar", vscrollbar);
	_window.addWidget("text_box", txt_hbox);
	_window.addWidget("text", text);
	
	GtkWidget *hbox = gtk_hbox_new(TRUE, 10);
	_window.addWidget("hbox", hbox);
	
	gtk_box_pack_start(GTK_BOX(hbox), close->getWidget(), FALSE, FALSE, 5);
	gtk_box_pack_start(GTK_BOX(hbox), save->getWidget(), FALSE, FALSE, 5);
	
	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 1);
	
	GtkWidget *win = _window.getWidget("toplevel");
	gtk_signal_connect(GTK_OBJECT(win), "destroy", GTK_SIGNAL_FUNC(ns_console::hide), NULL);
	gtk_signal_connect(GTK_OBJECT(win), "delete_event", GTK_SIGNAL_FUNC(ns_console::hide), NULL);	
	gtk_signal_connect(GTK_OBJECT(save->getWidget()), "clicked", GTK_SIGNAL_FUNC(ns_console::show_size_dialog), this);
	gtk_signal_connect_object(GTK_OBJECT(close->getWidget()), "clicked", GTK_SIGNAL_FUNC(ns_console::hide), GTK_OBJECT(win));

	//alocare culori
	_fatal = new GdkColor;
	_severe = new GdkColor;
	_major = new GdkColor;
	_minor = new GdkColor;
	_warning = new GdkColor;
	_normal = new GdkColor;
	_information = new GdkColor;
	
	setColor(_fatal, 255, 0, 0);	//rosu
	setColor(_severe, 255, 127, 0);	//portocaliu
	setColor(_major, 255, 0, 255);	//mov
	setColor(_minor, 0, 0, 255); //albastru
	setColor(_warning, 0, 180, 255); //bleu
	setColor(_normal, 0, 200, 0); //verde
	setColor(_information, 0, 0, 0); //negru
}

SizeW * Console::getSizeW()
{
	return &_sw;
}

void Console::setColor(GdkColor *color, int red, int green, int blue)
{
	color->red=red*(65535/255);
	color->green=green*(65535/255);
	color->blue=blue*(65535/255);
	color->pixel=(gulong)(red*65536 + green*256 + blue);
	gdk_colormap_alloc_color(gtk_widget_get_colormap(_window.getWidget("toplevel")), color, false, true);
}

int Console::size()
{
	return _lines;
}

void Console::setSize(int size)
{
	_lines = size;
}

void Console::addText(const string &str, GdkColor *fore)
{
	GtkWidget *text = _window.getWidget("text");
	gtk_text_freeze(GTK_TEXT(text));
	gtk_text_set_point(GTK_TEXT(text), 0);
	gtk_text_insert(GTK_TEXT(text), NULL, fore, NULL, (getDate() + " -- ").c_str(), -1);
	gtk_text_insert(GTK_TEXT(text), NULL, fore, NULL, str.c_str(), -1);
	int text_len = gtk_text_get_length(GTK_TEXT(text));
	if(text_len > _lines)
	{
		gtk_text_set_point(GTK_TEXT(text), _lines);
		gtk_text_forward_delete(GTK_TEXT(text), text_len - _lines);
	}
	gtk_text_thaw(GTK_TEXT(text));
	gtk_adjustment_set_value(GTK_ADJUSTMENT(GTK_TEXT(text)->vadj), 1);
}

void Console::show()
{
	_window.draw();
}

void Console::hide()
{
	gtk_widget_hide_all( _window.getWidget("toplevel") );
}

Console::~Console()
{
	gdk_colormap_free_colors(gtk_widget_get_colormap(_window.getWidget("toplevel")), _fatal, 1);
	gdk_colormap_free_colors(gtk_widget_get_colormap(_window.getWidget("toplevel")), _severe, 1);
	gdk_colormap_free_colors(gtk_widget_get_colormap(_window.getWidget("toplevel")), _major, 1);
	gdk_colormap_free_colors(gtk_widget_get_colormap(_window.getWidget("toplevel")), _minor, 1);
	gdk_colormap_free_colors(gtk_widget_get_colormap(_window.getWidget("toplevel")), _warning, 1);
	gdk_colormap_free_colors(gtk_widget_get_colormap(_window.getWidget("toplevel")), _normal, 1);
	gdk_colormap_free_colors(gtk_widget_get_colormap(_window.getWidget("toplevel")), _information, 1);
	delete _fatal;
	delete _severe;
	delete _major;
	delete _minor;
	delete _warning;
	delete _normal;
	delete _information;
}
